import numpy as np

from PyQt5 import QtWidgets

#  class: DataIO
#
#  This class contains methods to both save and load data for both
#  the Train and Eval windows of the application.
#
class DataIO:


    def __init__(self, ui ):
        self.ui = ui
        self.data = None

    def load_file(self, mode):
        self.mode = mode

        # prompts user for file - displayed at the top of pop-up when user
        # clicks on File menu, depends on file being train or eval
        #
        if self.mode is "train":
            file,_ = QtWidgets.QFileDialog.getOpenFileName(
            self.ui,
            "Load Train Data",
            "",
        )

        elif self.mode is "eval":
            file,_ = QtWidgets.QFileDialog.getOpenFileName(
            self.ui,
            "Load Eval Data",
            "",
        )

        else:
            pass

        # covert file into string for parsing
        #
        file = str(file)

        # make sure a file was selected
        #
        if file is None or len(file) == 0:
            return

        data = self.read_data(file)

        # read data
        #
        # check if data was read
        #
        if data is None:
            print("Warning: Data not read or updated")
            return False

        # remove colors from data:
        color = []
        for i in data:
            if len(i) == 1:
                color.append(i)
                data.remove(i)

        self.data = np.array(data)
        return self.data, color


    def save_file(self, data, mode):

        # prompts user for name to save file as, depending on data
        #
        if mode is 'train':
            save_name = QtWidgets.QFileDialog.getSaveFileName(self.ui,
                                                              "Save as...",
                                                              "/train_data.csv")
        elif mode is 'eval':
            save_name = QtWidgets.QFileDialog.getSaveFileName(self.ui,
                                                          "Save as...",
                                                          "/eval_data.csv")
        else:
            pass

        # make sure a file name was given
        #
        if save_name[0] is '' or len(save_name) == 0:
            return

        # creates the file and writes the data to it
        #
        with open(save_name[0], 'w') as fp:

            # import all class_info from InputDisplay
            #
            class_info = data
            for class_name in class_info:

                # retrieve class coordinate points
                #
                data_x = class_info[class_name][1]
                data_y = class_info[class_name][2]
                data_t = np.column_stack((data_x,data_y))
                fp.write(class_info[class_name][4])
                fp.write('\n')
                # write each point of each class in a csv file
                #
                print(data_t)
                for item in range(len(data_t)):
                    fp.write(class_name + ",")
                    for num in range(len(data_t[item])):
                        if data_t[item][num] == data_t[item][-1]:
                            fp.write("%8lf" % (data_t[item][num]))
                        else:
                            fp.write("%8lf," % (data_t[item][num]))
                    fp.write('\n')
        fp.close()

    @staticmethod
    def read_data(data):

        # loads file into memory
        #
        contents = [[x.strip() for x in line.split(',')] for line in open(data)]
        return contents

# end of class
#

# end of file
#